@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (C) 2014 J.F.Dockes
+# Copyright (C) 2014-2020 J.F.Dockes
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -20,17 +20,8 @@
 # pdftotext sometimes outputs unescaped text inside HTML text sections.
 # We try to correct.
 #
-# If pdftotext produces no text and tesseract is available, we try to
-# perform OCR. As this can be very slow and the result not always
-# good, we only do this if this is required by the configuration
-#
-# We guess the OCR language in order of preference:
-#  - From the content of a ".ocrpdflang" file if it exists in the same
-#    directory as the PDF
-#  - Else from the pdfocrlang in recoll.conf
-#  - Else from an RECOLL_TESSERACT_LANG environment variable
-#  - From the content of $RECOLL_CONFDIR/ocrpdf
-#  - Default to "eng"
+# If pdftotext produces no text and the configuration allows it, we may try to
+# perform OCR.
 
 from __future__ import print_function
 
@@ -47,8 +38,14 @@ import glob
 import traceback
 
 _mswindows = (sys.platform == "win32")
+    
 tmpdir = None
 
+_htmlprefix =b'''<html><head>
+<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">
+</head><body><pre>'''
+_htmlsuffix = b'''</pre></body></html>'''
+
 def finalcleanup():
     if tmpdir:
         vacuumdir(tmpdir)
@@ -120,18 +117,6 @@ class PDFExtractor:
         except:
             pass
         
-        # See if we'll try to perform OCR. Need the commands and the
-        # either the presence of a file in the config dir (historical)
-        # or a set config variable.
-        self.ocrpossible = False
-        self.tesseract = rclexecm.which("tesseract")
-        if self.tesseract:
-            self.pdftoppm = rclexecm.which("pdftoppm")
-            if self.pdftoppm:
-                self.ocrpossible = True
-                self.maybemaketmpdir()
-        # self.em.rclog("OCRPOSSIBLE: %d" % self.ocrpossible)
-
         # Pdftk is optionally used to extract attachments. This takes
         # a hit on performance even in the absence of any attachments,
         # so it can be disabled in the configuration.
@@ -236,100 +221,6 @@ class PDFExtractor:
         return (True, docdata, ipath, eof)
 
 
-    # Try to guess tesseract language. This should depend on the input
-    # file, but we have no general way to determine it. So use the
-    # environment and hope for the best.
-    def guesstesseractlang(self):
-        tesseractlang = ""
-
-        # First look for a language def file in the file's directory 
-        pdflangfile = os.path.join(os.path.dirname(self.filename),
-                                   b".ocrpdflang")
-        if os.path.isfile(pdflangfile):
-            tesseractlang = open(pdflangfile, "r").read().strip()
-        if tesseractlang:
-            return tesseractlang
-
-        # Then look for a global option. The normal way now that we
-        # have config reading capability in the handlers is to use the
-        # config. Then, for backwards compat, environment variable and
-        # file inside the configuration directory
-        tesseractlang = self.config.getConfParam("pdfocrlang")
-        if tesseractlang:
-            return tesseractlang
-        tesseractlang = os.environ.get("RECOLL_TESSERACT_LANG", "");
-        if tesseractlang:
-            return tesseractlang
-        pdflangfile = os.path.join(self.confdir, "ocrpdf")
-        if os.path.isfile(pdflangfile):
-            tesseractlang = open(pdflangfile, "r").read().strip()
-        if tesseractlang:
-            return tesseractlang
-
-        # Half-assed trial to guess from LANG then default to english
-        localelang = os.environ.get("LANG", "").split("_")[0]
-        if localelang == "en":
-            tesseractlang = "eng"
-        elif localelang == "de":
-            tesseractlang = "deu"
-        elif localelang == "fr":
-            tesseractlang = "fra"
-        if tesseractlang:
-            return tesseractlang
-
-        if not tesseractlang:
-            tesseractlang = "eng"
-        return tesseractlang
-
-    # PDF has no text content and tesseract is available. Give OCR a try
-    def ocrpdf(self):
-
-        global tmpdir
-        if not tmpdir:
-            return b""
-
-        tesseractlang = self.guesstesseractlang()
-        # self.em.rclog("tesseractlang %s" % tesseractlang)
-
-        tesserrorfile = os.path.join(tmpdir, "tesserrorfile")
-        tmpfile = os.path.join(tmpdir, "ocrXXXXXX")
-
-        # Split pdf pages
-        try:
-            vacuumdir(tmpdir)
-            subprocess.check_call([self.pdftoppm, "-r", "300", self.filename,
-                                   tmpfile])
-        except Exception as e:
-            self.em.rclog("pdftoppm failed: %s" % e)
-            return b""
-
-        files = glob.glob(tmpfile + "*")
-        for f in files:
-            out = b''
-            try:
-                out = subprocess.check_output([self.tesseract, f, f, "-l",
-                                               tesseractlang],
-                                              stderr = subprocess.STDOUT)
-            except Exception as e:
-                self.em.rclog("tesseract failed: %s" % e)
-
-            errlines = out.split(b'\n')
-            if len(errlines) > 2:
-                self.em.rclog("Tesseract error: %s" % out)
-
-        # Concatenate the result files
-        files = glob.glob(tmpfile + "*" + ".txt")
-        data = b""
-        for f in files:
-            data += open(f, "rb").read()
-
-        return b'''<html><head>
-        <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">
-        </head><body><pre>''' + \
-        self.em.htmlescape(data) + \
-        b'''</pre></body></html>'''
-
-
     # pdftotext (used to?) badly escape text inside the header
     # fields. We do it here. This is not an html parser, and depends a
     # lot on the actual format output by pdftotext.
@@ -439,14 +330,35 @@ class PDFExtractor:
         for metanm,rclnm in self.extrameta:
             for rdfdesc in rdfdesclist:
                 try:
-                    elt = rdfdesc.find(metanm, rdfdesc.nsmap)
+                    elts = rdfdesc.findall(metanm, rdfdesc.nsmap)
                 except:
                     # We get an exception when this rdf:Description does not
                     # define the required namespace.
                     continue
-                text = None
-                if elt is not None:
-                    text = self._xmltreetext(elt)
+                
+                if elts:
+                    for elt in elts:
+                        text = None
+                        try:
+                            # First try to get text from a custom element handler
+                            text = emf.metafixelt(metanm, elt)
+                        except:
+                            pass
+                        
+                        if text is None:
+                            # still nothing here, read the element text
+                            text = self._xmltreetext(elt)
+                            try:
+                                # try to run metafix
+                                text = emf.metafix(metanm, text)
+                            except:
+                                pass
+
+                        if text:
+                            # Can't use setfield as it only works for
+                            # text/plain output at the moment.
+                            #self.em.rclog("Appending: (%s,%s)"%(rclnm,text))
+                            metaheaders.append((rclnm, text))
                 else:
                     # Some docs define the values as attributes. don't
                     # know if this is valid but anyway...
@@ -456,17 +368,13 @@ class PDFExtractor:
                     except:
                         fullnm = metanm
                     text = rdfdesc.get(fullnm)
-                # Should we set empty values ?
-                if text:
-                    if emf:
+                    if text:
                         try:
+                            # try to run metafix
                             text = emf.metafix(metanm, text)
                         except:
                             pass
-                    # Can't use setfield as it only works for
-                    # text/plain output at the moment.
-                    #self.em.rclog("Appending: (%s,%s)"%(rclnm,text))
-                    metaheaders.append((rclnm, text))
+                        metaheaders.append((rclnm, text))
         if metaheaders:
             if emf:
                 try:
@@ -493,13 +401,18 @@ class PDFExtractor:
         html, isempty = self._fixhtml(html)
         #self.em.rclog("ISEMPTY: %d : data: \n%s" % (isempty, html))
 
-        if isempty and self.ocrpossible:
+        if isempty:
             self.config.setKeyDir(os.path.dirname(self.filename))
             s = self.config.getConfParam("pdfocr")
-            cf_doocr = rclexecm.configparamtrue(s)
-            file_doocr = os.path.isfile(os.path.join(self.confdir, "ocrpdf"))
-            if cf_doocr or file_doocr:
-                html = self.ocrpdf()
+            if rclexecm.configparamtrue(s):
+                try:
+                    cmd = [sys.executable, os.path.join(_execdir, "rclocr.py"),
+                           self.filename]
+                    data = subprocess.check_output(cmd)
+                    html = _htmlprefix + self.em.htmlescape(data) + _htmlsuffix
+                except Exception as e:
+                    self.em.rclog("%s failed: %s" % (cmd, e))
+                    pass
 
         if self.extrameta:
             try:
@@ -575,6 +488,7 @@ class PDFExtractor:
 
 
 # Main program: create protocol handler and extractor and run them
+_execdir = os.path.dirname(sys.argv[0])
 proto = rclexecm.RclExecM()
 extract = PDFExtractor(proto)
 rclexecm.main(proto, extract)
